"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OpenApiManagerV3_0 = void 0;
const openapi_types_1 = require("openapi-types");
const types_1 = require("../../utils/types");
require("@quinck/collections");
const openapi_manager_v3_0_consts_1 = require("./openapi-manager-v3_0.consts");
class OpenApiManagerV3_0 {
    generateOpenapiWithRoutesSchemas(openapi) {
        const methods = Object.values(openapi_types_1.OpenAPIV3.HttpMethods);
        if (openapi.paths === undefined)
            throw new Error('no paths');
        const paths = Object.values(openapi.paths)
            .singleCollect(path => path != undefined, path => path)
            .flatMap(path => methods.singleCollect(method => method in path, method => path[method]));
        function referenceToDesiredObject(componentName, { $ref }) {
            const componentId = $ref.replace(`#/components/${componentName}/`, '');
            if (openapi.components != undefined) {
                const component = openapi.components[componentName];
                if (component != undefined) {
                    const componentElement = component[componentId];
                    if (componentElement != undefined) {
                        if (isReferenceObject(componentElement))
                            return referenceToDesiredObject(componentName, componentElement);
                        return componentElement; //TODO find a way to remove this as
                    }
                }
            }
            throw new Error(); //TODO
        }
        function isReferenceObject(x) {
            return '$ref' in x;
        }
        function retrieveParameterObjects(params) {
            return params.singleCollect(isReferenceObject, param => referenceToDesiredObject('parameters', param), param => param);
        }
        function paramsToSchema(params) {
            if (params && params.length > 0) {
                const required = params.singleCollect(x => x.required === true, x => x.name);
                const properties = params.groupByToDictionary(x => x.name, x => x.schema);
                const result = {
                    ...openapi_manager_v3_0_consts_1.defaultObject,
                    properties,
                };
                if (required.length > 0)
                    result.required = required;
                return result;
            }
            return openapi_manager_v3_0_consts_1.defaultObject;
        }
        const pathsSchemas = paths.reduce((result, { operationId, parameters, requestBody, responses }) => {
            const params = retrieveParameterObjects(parameters ?? []);
            const paramsByScope = params.simpleGroupBy(x => x.in);
            const pathParamsSchema = paramsToSchema(paramsByScope.get('path'));
            const pathParamsSchemaName = `${operationId}${types_1.RouteTypeTag.PathParams}`;
            result[pathParamsSchemaName] = pathParamsSchema;
            const queryParamsSchema = paramsToSchema(paramsByScope.get('query'));
            const queryParamsSchemaName = `${operationId}${types_1.RouteTypeTag.QueryParams}`;
            result[queryParamsSchemaName] = queryParamsSchema;
            const bodySchemaName = `${operationId}${types_1.RouteTypeTag.RequestBody}`;
            result[bodySchemaName] = openapi_manager_v3_0_consts_1.defaultObject;
            if (requestBody != undefined) {
                const requestBodyObject = isReferenceObject(requestBody)
                    ? referenceToDesiredObject('requestBodies', requestBody)
                    : requestBody;
                const bodyMedias = Object.entries(requestBodyObject.content);
                if (bodyMedias.length > 0) {
                    const [[, bodyMediaObject]] = bodyMedias;
                    result[bodySchemaName] =
                        bodyMediaObject.schema ?? openapi_manager_v3_0_consts_1.defaultObject;
                }
            }
            const replySchemaName = `${operationId}${types_1.RouteTypeTag.ReplyBody}`;
            result[replySchemaName] = openapi_manager_v3_0_consts_1.defaultObject;
            if (responses) {
                const replySchemas = Object.values(responses)
                    .flatMap(response => {
                    const responseObject = isReferenceObject(response)
                        ? referenceToDesiredObject('responses', response)
                        : response;
                    const { content } = responseObject;
                    if (content) {
                        const medias = Object.values(content);
                        return medias.map(mediaObject => mediaObject.schema);
                    }
                    return [openapi_manager_v3_0_consts_1.nullObject];
                })
                    .singleCollect(schema => schema != undefined, schema => schema);
                if (replySchemas.length === 1)
                    result[replySchemaName] = replySchemas[0];
                else if (replySchemas.length > 1)
                    result[replySchemaName] = { oneOf: replySchemas };
            }
            return result;
        }, {});
        const newOpenapi = { ...openapi };
        if (!newOpenapi.components)
            newOpenapi.components = {};
        newOpenapi.components.schemas = {
            ...newOpenapi.components.schemas,
            ...pathsSchemas,
        };
        return newOpenapi;
    }
}
exports.OpenApiManagerV3_0 = OpenApiManagerV3_0;
